//
//  GameScene.swift
//  Project 11
//
//  Created by Michael Augustsson on 2018-10-12.
//  Copyright © 2018 Michael Augustsson. All rights reserved.
//

import SpriteKit
import GameplayKit

class GameScene: SKScene {
    
    var bubbleTextures = [SKTexture]()
    var currentBubbleTexture = 0
    var maximumNumber = 1
    var bubbles = [SKSpriteNode]()
    var bubbleTimer: Timer!
    var gameOverStatus = false
    
    override func didMove(to view: SKView) {
        
        bubbleTextures.append(SKTexture(imageNamed: "bubbleBlue"))
        bubbleTextures.append(SKTexture(imageNamed: "bubbleCyan"))
        bubbleTextures.append(SKTexture(imageNamed: "bubbleGray"))
        bubbleTextures.append(SKTexture(imageNamed: "bubbleGreen"))
        bubbleTextures.append(SKTexture(imageNamed: "bubbleOrange"))
        bubbleTextures.append(SKTexture(imageNamed: "bubblePink"))
        bubbleTextures.append(SKTexture(imageNamed: "bubblePurple"))
        bubbleTextures.append(SKTexture(imageNamed: "bubbleRed"))
        
        physicsBody = SKPhysicsBody(edgeLoopFrom: frame)
        physicsWorld.gravity = CGVector.zero
        
        for _ in 1...8 {
            
            createBubble()
        }
        
        bubbleTimer = Timer.scheduledTimer(timeInterval: 3, target: self, selector: #selector(createBubble), userInfo: nil, repeats: true)
    }
    
    @objc func createBubble() {
        
        // 1 - create a new sprite node from our current texture
        let bubble = SKSpriteNode(texture: bubbleTextures[currentBubbleTexture])
        
        // 2 - give it the stringified version of our current number
        bubble.name = String(maximumNumber)
        
        // 3 - gove it a z-position of 1, so it draws above any background
        bubble.zPosition = 1
        
        // 4 - create a label node with the current number, in nice, brigh text
        let label = SKLabelNode(fontNamed: "HelveticaNeue-Light")
        label.text = bubble.name
        label.color = NSColor.white
        label.fontSize = 64
        
        // 5 - make the label center itself vertically and draw above the bubble
        label.verticalAlignmentMode = .center
        label.zPosition = 2
        
        // 6 - add the label to the bubble, then the bubble to the game scene
        bubble.addChild(label)
        addChild(bubble)
        
        // 7 - add the new bubble to our array for later use
        bubbles.append(bubble)
        
        // 8 - make it appear somewhere inside our game scen
        let xPos = GKRandomSource.sharedRandom().nextInt(upperBound: 800)
        let yPos = GKRandomSource.sharedRandom().nextInt(upperBound: 600)
        
        bubble.position = CGPoint(x: xPos, y: yPos)
        
        let scale = CGFloat(GKRandomSource.sharedRandom().nextUniform())
        bubble.setScale(max(0.7, scale))
        
        bubble.alpha = 0
        bubble.run(SKAction.fadeIn(withDuration: 0.5))
        
        configurePhysics(for: bubble)
        nextBubble()
    }
    
    func nextBubble() {
        
        // move on to the next bubble texture
        currentBubbleTexture += 1
        
        // if we've used all the bubble texture, start at the beginning
        if currentBubbleTexture == bubbleTextures.count {
            currentBubbleTexture = 0
        }
        
        // add a random number between q and 3 to maximumNumber
        maximumNumber += GKRandomSource.sharedRandom().nextInt(upperBound: 3) + 1
        
        // fix the mystery problem
        let strMaximumNumber = String(maximumNumber)
    
        if strMaximumNumber.last! == "6" {
            maximumNumber += 1
        }
        
        if strMaximumNumber.last! == "9" {
            maximumNumber += 1
        }
    }
    
    func configurePhysics(for bubble: SKSpriteNode) {
        
        bubble.physicsBody = SKPhysicsBody(circleOfRadius: bubble.size.width / 2)
        bubble.physicsBody?.linearDamping = 0.0
        bubble.physicsBody?.angularDamping = 0.0
        bubble.physicsBody?.restitution = 1.0
        bubble.physicsBody?.friction = 0.0
        
        let motionX = GKRandomSource.sharedRandom().nextInt(upperBound: 400) - 200
        let motionY = GKRandomSource.sharedRandom().nextInt(upperBound: 400) - 200
        
        bubble.physicsBody?.velocity = CGVector(dx: motionX, dy: motionY)
        bubble.physicsBody?.angularVelocity = CGFloat(GKRandomSource.sharedRandom().nextUniform())
    }
    
    func pop(_ node: SKSpriteNode) {
        
        guard let index = bubbles.index(of: node) else { return }
        bubbles.remove(at: index)
        
        node.physicsBody = nil
        node.name = nil
        
        let fadeOut = SKAction.fadeOut(withDuration: 0.3)
        let scaleUp = SKAction.scale(by: 1.5, duration: 0.3)
        scaleUp.timingMode = .easeOut
        let group = SKAction.group([fadeOut, scaleUp])
        
        let sequence = SKAction.sequence([group, SKAction.removeFromParent()])
        node.run(sequence)
        
        run(SKAction.playSoundFileNamed("pop.wav", waitForCompletion: false))
        
        if bubbles.count == 0 {
            bubbleTimer.invalidate()
            gameOver()
        }
    }
    
    override func mouseDown(with event: NSEvent) {
        
        // If we are at the Game Over screen then restart everything
        if gameOverStatus {

            resetRestartGame()
            
            return
        }
        
        // find where we clicked in SpriteKit
        let location = event.location(in: self)
        
        // filter oiut nodes that don't have a name
        let clickNodes = nodes(at: location).filter { $0.name != nil }
        
        // make sure at least one clicked node remains
        guard clickNodes.count != 0 else { return }
        
        // find the lowest-numbered bubble on the screen
        let lowestBubble = bubbles.min { Int($0.name!)! < Int($1.name!)! }
        guard let bestNumber = lowestBubble?.name else { return }
        
        // go through all nodes the user clicked to see if any of them is the best number
        for node in clickNodes {
            if node.name == bestNumber {
                // they where correct - pop the bubble!
                pop(node as! SKSpriteNode)
                
                // exit the method so we don't create new bubble
                return
            }
        }
        
        // if we're still here it means they were incorrecy; create two penalty bubble
        createBubble()
        createBubble()
    }
    
    func gameOver() {
        
        createLetterBubble(letter: "G", xpos: 50, ypos: 300, texture: GKRandomSource.sharedRandom().nextInt(upperBound: 8))
        createLetterBubble(letter: "A", xpos: 140, ypos: 300, texture: GKRandomSource.sharedRandom().nextInt(upperBound: 8))
        createLetterBubble(letter: "M", xpos: 230, ypos: 300, texture: GKRandomSource.sharedRandom().nextInt(upperBound: 8))
        createLetterBubble(letter: "E", xpos: 320, ypos: 300, texture: GKRandomSource.sharedRandom().nextInt(upperBound: 8))
        createLetterBubble(letter: "O", xpos: 483, ypos: 300, texture: GKRandomSource.sharedRandom().nextInt(upperBound: 8))
        createLetterBubble(letter: "V", xpos: 573, ypos: 300, texture: GKRandomSource.sharedRandom().nextInt(upperBound: 8))
        createLetterBubble(letter: "E", xpos: 663, ypos: 300, texture: GKRandomSource.sharedRandom().nextInt(upperBound: 8))
        createLetterBubble(letter: "R", xpos: 753, ypos: 300, texture: GKRandomSource.sharedRandom().nextInt(upperBound: 8))
        run(SKAction.playSoundFileNamed("fanfare.wav", waitForCompletion: true))
        
        gameOverStatus = true
    }
    
    func createLetterBubble(letter: String, xpos: Int, ypos: Int, texture: Int) {
        
        let bubble = SKSpriteNode(texture: bubbleTextures[texture])
        let label = SKLabelNode(fontNamed: "HelveticaNeue-Light")
        bubble.name = letter
        label.text = letter
        label.color = NSColor.white
        label.fontSize = 64
        label.verticalAlignmentMode = .center
        label.zPosition = 2
        bubble.addChild(label)
        addChild(bubble)
        bubble.position = CGPoint(x: xpos, y: ypos)
        bubble.setScale(0.6)
        
        
    }
    
    func resetRestartGame() {
        
        // Remove the GAME OVER Bubbles
        self.removeAllChildren()
        
        // Reset all game data
        currentBubbleTexture = 0
        maximumNumber = 1
        gameOverStatus = false
        
        // Create 8 bubbles to start with
        for _ in 1...8 {
            
            createBubble()
        }
        
        // and get the timmer going again
        bubbleTimer = Timer.scheduledTimer(timeInterval: 3, target: self, selector: #selector(createBubble), userInfo: nil, repeats: true)
        
        
    }
    
}
